#include <stdio.h>
#include <stdlib.h>

#define MAXN 1000
static int gr[MAXN][MAXN];
static int ns[MAXN][MAXN];
static int deg[MAXN];

static void
clear_graph (int n)
{
  int i, j;

  for (i = 0; i < n; i++)
    {
      deg[i] = 0;
      for (j = 0; j < n; j++)
	gr[i][j] = 0;
    }
}

static void
print_graph (FILE *f, int n)
{
  int i, j, m = 0;
  int ren[MAXN];

  for (i = 0; i < n; i++)
    ren[i] = i;
  for (i = 0; i < n; i++)
    {
      int t;

      j = i + rand () % (n - i);
      t = ren[i];
      ren[i] = ren[j];
      ren[j] = t;
    }

  for (i = 0; i < n; i++)
    m += deg[i];
  m /= 2;

  fprintf (f, "%d %d\n", n, m);
  for (i = 0; i < n; i++)
    {
      int v = ren[i];

      for (j = 0; j < deg[v]; j++)
	if (ns[v][j] > v)
	  fprintf (f, "%d %d\n", ren[v] + 1, ren[ns[v][j]] + 1);
    }
}

static void
add (int u, int v)
{
  if (u ==v || gr[u][v])
    return;

  gr[u][v] = gr[v][u] = 1;
  ns[u][deg[u]++] = v;
  ns[v][deg[v]++] = u;
}

static void
randgr (FILE *f, int n)
{
  int i, j;

  clear_graph (n);
  for (i = 0; i < n; i++)
    for (j = i + 1; j < n; j++)
      if (rand () % 2)
	add (i, j);

  print_graph (f, n);
}

static int
random_max_clique (int n, int cl[])
{
  int v = rand () % n;
  int sz = 1;
  int vs[MAXN];
  int i, j, d;

  cl[0] = v;
  d = deg[v];
  for (i = 0; i < d; i++)
    vs[i] = ns[v][i];

  for (i = 0; i < d; i++)
    {
      int ew = i + rand () % (d - i);
      int t = vs[i];
      vs[i] = vs[ew];
      vs[ew] = t;
    }

  for (i = 0; i < d; i++)
    {
      for (j = 0; j < sz; j++)
	if (!gr[cl[j]][vs[i]])
	  break;
      if (j == sz)
	cl[sz++] = vs[i];
    }

  return sz;
}

static void
chordal (int n, int mc)
{
  int i, j;
  int cl[MAXN], cs;

  for (i = 1; i < n; i++)
    {
      cs = random_max_clique (i, cl);
      if (cs > mc)
	cs = mc;

      for (j = 0; j < cs; j++)
	add (i, cl[j]);
    }
}

static int
ind_cycle_len (int u, int v, int n)
{
  int dist[MAXN], queue[MAXN];
  int i, qb, ql;

  for (i = 0; i < n; i++)
    dist[i] = -1;

  for (i = 0; i < deg[u]; i++)
    if (gr[ns[u][i]][v])
      dist[ns[u][i]] = 0;

  qb = 0;
  ql = 1;
  queue[0] = u;
  dist[u] = 0;

  while (ql)
    {
      u = queue[qb++];
      ql--;

      if (u == v)
	return dist[v] + 1;

      for (i = 0; i < deg[u]; i++)
	{
	  int x = ns[u][i];

	  if (dist[x] == -1)
	    {
	      dist[x] = dist[u] + 1;
	      queue[qb + ql++] = x;
	    }
	}
    }

  return -1;
}

static void
chord_plus_edge (FILE *f, int n, int mc, int ml, int pe)
{
  int u, v;

  clear_graph (n);

  chordal (n, mc);

  if (pe)
    {
      while (1)
	{
	  u = rand () % n;
	  v = rand () % n;

	  if (u == v || gr[u][v])
	    continue;

	  if (ind_cycle_len (u, v, n) >= ml)
	    {
	      add (u, v);
	      break;
	    }
	}
    }

  print_graph (f, n);
}

static int
creates_le5cycle (int u, int v, int n)
{
  int dist[MAXN], queue[MAXN];
  int i, qb, ql;

  for (i = 0; i < n; i++)
    dist[i] = -1;

  qb = 0;
  ql = 1;
  queue[0] = u;
  dist[u] = 0;

  while (ql)
    {
      u = queue[qb++];
      ql--;

      if (u == v)
	return dist[v] <= 4;

      for (i = 0; i < deg[u]; i++)
	{
	  int x = ns[u][i];

	  if (dist[x] == -1)
	    {
	      dist[x] = dist[u] + 1;
	      queue[qb + ql++] = x;
	    }
	}
    }

  return 0;
}

static void
rand_girth5 (FILE *f, int n, int m)
{
  int u, v;
  int ncc;

  clear_graph (n);

  ncc = 0;
  while (m)
    {
      if (ncc > 10000)
	break;
      ncc++;

      u = rand () % n;
      v = rand () % n;

      if (u == v || gr[u][v])
	continue;

      if (creates_le5cycle (u, v, n))
	continue;

      add (u, v);
      m--;
      ncc = 0;
    }

  for (u = 0; u < n; u++)
    for (v = u + 1; v < n; v++)
      if (m && !creates_le5cycle (u, v, n))
	{
	  add (u, v);
	  m--;
	}

  print_graph (f, n);
}

static void
path_like (FILE *f, int n, int cl, int ae)
{
  int i, j;

  clear_graph (n);
  for (i = cl; i < n; i++)
    for (j = i - cl; j < i; j++)
      add (i, j);

  if (ae)
    add (n / 2 - 2 * cl, n / 2 + 2 * cl);

  print_graph (f, n);
}

static void
reduced_comp (FILE *f, int n, int hs, int redu)
{
  int i, j;
  int mih = n / 2 - hs, mah = n / 2 + hs;
  clear_graph (n);

  for (i = 0; i < n; i++)
    for (j = i + 1; j < n; j++)
      {
	if (redu
	    && i >= mih && i <= mah
	    && j >= mih && j <= mah
	    && j != i + 1
	    && (i != mih || j != mah))
	  continue;
	add (i, j);
      }

  print_graph (f, n);
}

static void
funnel (FILE *f, int n, int addc)
{
  int i, j, cs = n / 6;
  clear_graph (n);

  for (i = 0; i < cs; i++)
    for (j = i + 1; j < cs; j++)
      {
	add (i, j);
	add (n - 1 - i, n - 1 - j);
      }

  for (i = cs; i < n / 2; i++)
    {
      for (j = 1; j < (n / 2 - i) / 3; j++)
	{
	  add (i, j);
	  add (n - 1 - i, n - 1 - j);
	}

      add (i, i - 1);
      add (n - 1 - i, n - i);
    }
  add (n / 2, n / 2 - 1);

  if (addc)
    add (n / 2 - 15, n / 2 + 10);

  print_graph (f, n);
}

int main (void)
{
  FILE *f;

  fprintf (stderr, "GEN 1\n");
  f = fopen ("data/indcyc.1a", "w");
  fprintf (f, "4 4\n1 2\n2 3\n3 4\n4 1\n");
  fclose (f);
  f = fopen ("data/indcyc.1b", "w");
  fprintf (f, "4 4\n1 2\n2 3\n3 1\n4 1\n");
  fclose (f);
  f = fopen ("data/indcyc.1c", "w");
  fprintf (f, "4 0\n");
  fclose (f);
  f = fopen ("data/indcyc.1d", "w");
  fprintf (f, "2 1\n1 2\n");
  fclose (f);
  f = fopen ("data/indcyc.1e", "w");
  fprintf (f, "0 0\n");
  fclose (f);

  fprintf (stderr, "GEN 2\n");
  f = fopen ("data/indcyc.2", "w");
  srand (2);
  randgr (f, 10);
  fclose (f);

  fprintf (stderr, "GEN 3\n");
  f = fopen ("data/indcyc.3a", "w");
  srand (3);
  chord_plus_edge (f, 10, 3, 4, 1);
  fclose (f);

  f = fopen ("data/indcyc.3b", "w");
  srand (31);
  chord_plus_edge (f, 10, 3, 4, 0);
  fclose (f);

  fprintf (stderr, "GEN 4\n");
  f = fopen ("data/indcyc.4", "w");
  srand (4);
  rand_girth5(f, 100, 1000);
  fclose (f);

  fprintf (stderr, "GEN 5\n");
  f = fopen ("data/indcyc.5a", "w");
  srand (5);
  chord_plus_edge (f, 100, 10, 6, 1);
  fclose (f);

  f = fopen ("data/indcyc.5b", "w");
  srand (51);
  chord_plus_edge (f, 100, 10, 6, 0);
  fclose (f);

  fprintf (stderr, "GEN 6\n");
  f = fopen ("data/indcyc.6a", "w");
  path_like(f, 300, 10, 0);
  fclose (f);

  f = fopen ("data/indcyc.6b", "w");
  path_like(f, 300, 10, 1);
  fclose (f);

  f = fopen ("data/indcyc.6c", "w");
  funnel (f, 300, 1);
  fclose (f);

  f = fopen ("data/indcyc.6d", "w");
  funnel (f, 300, 0);
  fclose (f);

  fprintf (stderr, "GEN 7\n");
  f = fopen ("data/indcyc.7a", "w");
  srand (7);
  chord_plus_edge (f, 300, 10, 8, 1);
  fclose (f);

  f = fopen ("data/indcyc.7b", "w");
  srand (71);
  chord_plus_edge (f, 300, 10, 8, 0);
  fclose (f);

  fprintf (stderr, "GEN 8\n");
  f = fopen ("data/indcyc.8a", "w");
  path_like(f, 1000, 50, 1);
  fclose (f);

  f = fopen ("data/indcyc.8b", "w");
  path_like(f, 1000, 20, 1);
  fclose (f);

  f = fopen ("data/indcyc.8c", "w");
  path_like(f, 1000, 50, 0);
  fclose (f);

  f = fopen ("data/indcyc.8d", "w");
  path_like(f, 1000, 20, 0);
  fclose (f);

  fprintf (stderr, "GEN 9\n");
  f = fopen ("data/indcyc.9a", "w");
  srand (9);
  chord_plus_edge (f, 1000, 20, 7, 1);
  fclose (f);

  f = fopen ("data/indcyc.9b", "w");
  srand (91);
  chord_plus_edge (f, 1000, 20, 7, 0);
  fclose (f);

  fprintf (stderr, "GEN 10\n");
  f = fopen ("data/indcyc.10a", "w");
  reduced_comp (f, 440, 6, 1);
  fclose (f);

  f = fopen ("data/indcyc.10b", "w");
  reduced_comp (f, 440, 6, 0);
  fclose (f);

  f = fopen ("data/indcyc.10c", "w");
  funnel (f, 1000, 1);
  fclose (f);

  f = fopen ("data/indcyc.10d", "w");
  funnel (f, 1000, 0);
  fclose (f);

  return 0;
}
