#include <stdio.h>
#include <stdlib.h>

#define MAXN 1000

static int g[MAXN][MAXN];
static int seq[MAXN];
static int seen[MAXN];

#define BUG(...) do { fprintf(stderr, "Internal error: " __VA_ARGS__); exit(1); } while (0)
#define WRONG(...) do { fprintf(stderr, "Wrong answer\n"); puts("0"); exit(0); } while (0)
#define OKAY(...) do { fprintf(stderr, "Correct\n"); puts("1"); exit(0); } while (0)

int main (int argc, char *argv[])
{
  if (argc != 4)
    {
      fprintf(stderr, "Usage: %s <input> <correct-output> <checked-output>\n", argv[0]);
      return 1;
    }

  FILE *fin = fopen (argv[1], "r");
  FILE *cor = fopen (argv[2], "r");
  FILE *check = fopen (argv[3], "r");
  int solvable, n, m, i, j, u, v, s;

  solvable = (fscanf (cor, "%d", &i) == 1);

  fscanf (fin, "%d%d", &n, &m);
  for (i = 0; i < m; i++)
    {
      fscanf (fin, "%d%d", &u, &v);
      u--; v--;
      g[u][v] = g[v][u] = 1;
    }

  if (!solvable)
    {
      char c;

      if (fscanf (check, " %c", &c) != 1)
	WRONG("Empty output\n");

      if (c != 'n')
	WRONG("Wrong answer on graph without induced cycle\n");

      OKAY();
    }

  s = 0;
  while (fscanf (check, "%d", &v) == 1)
    {
      if (v < 1 || v > n)
	WRONG("Out of bound vertex number\n");
      v--;
      if (seen[v])
	WRONG("Repeated vertex\n");
      seen[v] = 1;
      seq[s++] = v;
    }
  if (s <= 3)
    WRONG("Too short sequence\n");

  for (i = 0; i < s; i++)
    for (j = i + 1; j < s; j++)
      {
	int sa;

	u = seq[i];
	v = seq[j];

	sa = (j == i + 1) || (i == 0 && j == s - 1);

	if (g[u][v] != sa)
	  WRONG("Wrong adjacency\n");
      }

  OKAY();
}
