#include <stdio.h>
#include <stdlib.h>

#define MAXN 1000000

struct edge { int x, y; };
struct edge ok[MAXN], his[MAXN];
int nok, nhis;

#define BUG(...) do { fprintf(stderr, "Internal error: " __VA_ARGS__); exit(1); } while (0)
#define WRONG(...) do { fprintf(stderr, "Wrong answer\n"); puts("0"); exit(0); } while (0)
#define OKAY(...) do { fprintf(stderr, "Correct\n"); puts("1"); exit(0); } while (0)

int rd(char *name, struct edge *e)
{
	int n = 0;

	FILE *f = fopen(name, "r");
	if (!f)
		BUG("Cannot open %s: %m\n", name);

	int x, y;
	while (fscanf(f, "%d%d", &x, &y) == 2) {
		if (n >= MAXN)
			return -1;
		if (x < y)
			e->x=x, e->y=y;
		else
			e->x=y, e->y=x;
		n++;
		e++;
	}

	fclose(f);
	return n;
}

#define COMPARE(x,y) do { if ((x)<(y)) return -1; if ((x)>(y)) return 1; } while(0)
int ecmp(const void *A, const void *B)
{
	const struct edge *a = A;
	const struct edge *b = B;
	COMPARE(a->x, b->x);
	COMPARE(a->y, b->y);
	return 0;
}

int main(int argc, char **argv)
{
	if (argc != 4) {
		fprintf(stderr, "Usage: %s <input> <correct-output> <checked-output>\n", argv[0]);
		return 1;
	}

	nok = rd(argv[2], ok);
	if (nok < 0)
		BUG("Too many edges in input file\n");

	nhis = rd(argv[3], his);
	if (nhis < 0)
		WRONG("Too many edges in output file\n");
	if (nhis != nok)
		WRONG("Wrong number of edges\n");

	qsort(ok, nok, sizeof(struct edge), ecmp);
	qsort(his, nhis, sizeof(struct edge), ecmp);

	for (int i=0; i<nok; i++)
		if (ok[i].x != his[i].x || ok[i].y != his[i].y)
			WRONG("Mismatched edge\n");

	OKAY();
}
