#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define N 10000007
#define M (10000000ull * 10000000ull)
#define R 1459612960807ull

static int seen[N][2];

static int
was_seen (unsigned long long x, unsigned long long y)
{
  unsigned long long v = x + 10000000 * y;
  unsigned long long comb = ((v * R) % M) % N;
  int p = comb;

  while (seen[p][0])
    {
      if (seen[p][0] == x && seen[p][1] == y)
	return 1;

      p++;
      if (p == N)
	p = 0;
    }

  seen[p][0] = x;
  seen[p][1] = y;
  
  return 0;
}

static void
fail (char *fn)
{
  fprintf (stderr, "Chyba v %s\n", fn);
  abort ();
}

static void
verify_constraints (int testno, long long n, long long w, long long h, long long q, char *fn)
{
  switch (testno)
    {
    case 0:
      if (h != 1 || n * w > 100000000 || q * w > 100000000)
	fail (fn);
      break;
    case 1:
      if (n * w * h > 100000000 || q * w * h > 100000000)
	fail (fn);
      break;
    case 2:
      if (h != 1 || n * w > 100000000)
	fail (fn);
      break;
    case 3:
      if (n * w * h > 100000000)
	fail (fn);
      break;
    case 4:
      if (h != 1)
	fail (fn);
      break;
    case 5:
    case 6:
      break;

    default:
      fail (fn);
    }
}

int main (int argc, char *argv[])
{
  char *fn = argv[1], *b;
  int testno;
  FILE *f;
  int i, n, q, w, h;

  b = strchr (fn, '.') + 1;
  testno = atoi (b) - 1;

  if (testno < 0 || testno > 13)
    fail (fn);

  f = fopen (fn, "r");
  if (fscanf (f, "%d%d", &w, &h) != 2)
    fail (fn);

  if (w <= 0 || h <= 0 || (long long) w * h > 2500000)
    fail (fn);

  if (fscanf (f, "%d", &n) != 1)
    fail (fn);

  if (n <= 0 || n > 200000)
    fail (fn);

  for (i = 0; i < n; i++)
    {
      int x, y, a, b;

      if (fscanf (f, "%d%d%d%d", &x, &y, &a, &b) != 4)
	fail (fn);

      if (x <= 0 || x > w
	  || y <= 0 || y > h
	  || a <= 0 || a > 1000000000
	  || b <= 0 || b > 1000000000)
	fail (fn);

      if (was_seen (x, y))
	fail (fn);

      if (testno / 2 == 5)
	{
	  int dist = (a - 1) / b;

	  if (x - dist <= 0 || x + dist > w
	      || y - dist <= 0 || y + dist > h)
	    fail (fn);
	}
    }

  if (fscanf (f, "%d", &q) != 1)
    fail (fn);

  if (q <= 0 || q > 200000)
    fail (fn);

  verify_constraints (testno / 2, n, w, h, q, fn);

  for (i = 0; i < q; i++)
    {
      int x1, y1, x2, y2;

      if (fscanf (f, "%d%d%d%d", &x1, &y1, &x2, &y2) != 4)
	fail (fn);

      if (x1 <= 0 || x1 > x2 || x2 > w)
	fail (fn);
      if (y1 <= 0 || y1 > y2 || y2 > h)
	fail (fn);
    }

  if (fscanf (f, "%d", &i) == 1)
    fail (fn);

  return 0;
}
